@@ -56,6 +56,12 @@
 
             cputime/1,				% -Seconds
             walltime/1,				% -Seconds
+            timed_call/2,                       % :Goal, :Options
+
+            debug_ctl/2,                        % +Option, +Value
+
+            fmt_write/2,                        % +Fmt, +Term
+            fmt_write/3,                        % +Stream, +Fmt, +Term
 
             path_sysop/2,                       % +Op, ?Value
             path_sysop/3,                       % +Op, ?Value1, ?Value2
@@ -74,6 +80,7 @@
 :- use_module(library(debug)).
 :- use_module(library(dialect/xsb/source)).
 :- use_module(library(dialect/xsb/tables)).
+:- use_module(library(dialect/xsb/timed_call)).
 :- use_module(library(aggregate)).
 :- use_module(library(option)).
 :- use_module(library(apply)).
@@ -370,7 +377,8 @@ load_dyn(File)       :-
         style_check(-singleton),
         load_files(File),
         '$style_check'(_, Style)).
-        load_dyn(File, Dir)  :- must_be(oneof([z]), Dir), load_dyn(File).
+
+load_dyn(File, Dir)  :- must_be(oneof([z]), Dir), load_dyn(File).
 load_dync(File)      :- load_dyn(File).
 load_dync(File, Dir) :- load_dyn(File, Dir).
 
@@ -510,6 +518,79 @@ walltime(Seconds) :-
     statistics(epoch, Epoch),
     Seconds is Now - Epoch.
 
+%!  debug_ctl(+Option, +Value) is det.
+%
+%   Control the XSB debugger. The  current implementation merely defines
+%   the predicate. Much more can be mapped to SWI-Prolog primitives.
+
+debug_ctl(prompt, off) :-
+    !,
+    leash(-all).
+debug_ctl(prompt, on) :-
+    !,
+    leash(+full).
+debug_ctl(hide, Preds) :-
+    !,
+    '$hide'(Preds).
+debug_ctl(Option, Value) :-
+    debug(xsb(compat), 'XSB: not implemented: ~p',
+          [ debug_ctl(Option, Value) ]).
+
+%!  fmt_write(+Fmt, +Term) is det.
+%!  fmt_write(+Stream, +Fmt, +Term) is det.
+%
+%   C-style formatted write, where  the  arguments   are  formed  by the
+%   arguments of Term.  We map this to format/2,3.
+%
+%   @bug We need to complete the  translation of the fmt_write sequences
+%   to format/2,3 sequences. Probably we should   also  cache the format
+%   translation.
+
+fmt_write(Fmt, Term) :-
+    fmt_write(current_output, Fmt, Term).
+
+fmt_write(Stream, Fmt, Term) :-
+    (   compound(Term)
+    ->  Term =.. [_|Args]
+    ;   Args = [Term]
+    ),
+    string_codes(Fmt, Codes),
+    phrase(format_fmt(Format, []), Codes),
+    format(Stream, Format, Args).
+
+format_fmt(Format, Tail) -->
+    "%",
+    (   format_esc(Format, Tail0)
+    ->  !
+    ;   here(Rest),
+        { print_message(warning, xsb(fmt_write(ignored(Rest)))),
+          fail
+        }
+    ),
+    format_fmt(Tail0, Tail).
+format_fmt([0'~,0'~|T0], T) -->
+    "~",
+    !,
+    format_fmt(T0, T).
+format_fmt([H|T0], T) -->
+    [H],
+    !,
+    format_fmt(T0, T).
+format_fmt(T, T) --> [].
+
+format_esc(Fmt, Tail) -->
+    format_esc(Fmt0),
+    !,
+    { append(Fmt0, Tail, Fmt)
+    }.
+
+format_esc(`~16r`) --> "x".
+format_esc(`~d`) --> "d".
+format_esc(`~f`) --> "f".
+format_esc(`~s`) --> "s".
+format_esc(`%`) --> "%".
+
+here(Rest, Rest, Rest).
 
 %!  path_sysop(+Op, ?Value) is semidet.
 %!  path_sysop(+Op, ?Arg1, ?Arg2) is semidet.
@@ -590,6 +671,8 @@ path_sysop(expand, Name, Path) :-
     prolog:message//1.
 
 prolog:message(xsb(not_in_module(File, Module, PI))) -->
-    [ '~p, implementing ~p does not export ~p'-[File, Module, PI] ].
+    [ 'XSB: ~p, implementing ~p does not export ~p'-[File, Module, PI] ].
 prolog:message(xsb(file_loaded_into_mismatched_module(File, Module))) -->
-    [ 'File ~p defines module ~p'-[File, Module] ].
+    [ 'XSB: File ~p defines module ~p'-[File, Module] ].
+prolog:message(xsb(ignored(debug_ctl(Option, Value)))) -->
+    [ 'XSB: debug_ctl(~p,~p) is not implemented'-[Option,Value] ].
