@@ -34,6 +34,8 @@
 
 :- module(increval,
           [ incr_assert/1,                      % :Clause
+            incr_asserta/1,                     % :Clause
+            incr_assertz/1,                     % :Clause
             incr_retractall/1,                  % :Head
             incr_retract/1,                     % :Clause
 
@@ -63,6 +65,8 @@ corresponding database update.
 
 :- meta_predicate
     incr_assert(:),
+    incr_asserta(:),
+    incr_assertz(:),
     incr_retractall(:),
     incr_retract(:),
     is_incremental_subgoal(:),
@@ -73,6 +77,8 @@ corresponding database update.
     incr_invalidate_calls(:).
 
 incr_assert(T)     :- assertz(T).
+incr_asserta(T)    :- asserta(T).
+incr_assertz(T)    :- assertz(T).
 incr_retractall(T) :- retractall(T).
 incr_retract(T)    :- retract(T).
 
