@@ -147,6 +147,24 @@ in case of failure or exceptions.
           ...
   ==
 
+## Socket exceptions			{#socket-exceptions}
+
+Errors that are trapped by  the  low-level   library  are  mapped  to an
+exception of the shape below. In this term,  `Code` is a lower case atom
+that corresponds to the C macro name,   e.g., `epipe` for a broken pipe.
+`Message` is the human readable string for   the  error code returned by
+the OS or  the  same  as  `Code`  if   the  OS  does  not  provide  this
+functionality. Note that `Code` is derived from   a static set of macros
+that may or may not be defines for the   target OS. If the macro name is
+not known, `Code` is =|ERROR_nnn|=, where _nnn_ is an integer.
+
+    error(socket_error(Code, Message), _)
+
+Note that on Windows `Code` is a ``wsa*``   code  which makes it hard to
+write portable code that handles specific   socket errors. Even on POSIX
+systems the exact set of errors  produced   by  the network stack is not
+defined.
+
 ## TCP socket predicates                {#socket-predicates}
 */
 
@@ -378,15 +396,17 @@ tcp_connect_direct(Address, Socket, StreamPair):-
 
 %!  tcp_select(+ListOfStreams, -ReadyList, +TimeOut)
 %
-%   Same as the built-in  wait_for_input/3,   but  integrates better
-%   with event processing and the  various   options  of sockets for
-%   Windows.   On   non-windows   systems     this    simply   calls
-%   wait_for_input/3.
+%   Same as the built-in wait_for_input/3. Used  to allow for interrupts
+%   and timeouts on Windows. A redesign  of the Windows socket interface
+%   makes  it  impossible  to  do  better  than  Windows  select()  call
+%   underlying wait_for_input/3. As input multiplexing typically happens
+%   in a background thread anyway we  accept   the  loss of timeouts and
+%   interrupts.
+%
+%   @deprecated Use wait_for_input/3
 
-:- if(\+predicate_property(tcp_select(_,_,_), defined)).
 tcp_select(ListOfStreams, ReadyList, TimeOut) :-
     wait_for_input(ListOfStreams, ReadyList, TimeOut).
-:- endif.
 
 
                  /*******************************
@@ -526,6 +546,14 @@ try_proxy(socks(Host, Port), Address, Socket, StreamPair) :-
 %     dispatched on behalf of the user interface. Default is
 %     =true=. Only very specific situations require setting
 %     this to =false=.
+%
+%     - sndbuf(+Integer)
+%     Sets the send buffer size to Integer (bytes). On Windows this defaults
+%     (now) to 64kb. Higher latency links may benefit from increasing this
+%     further since the maximum theoretical throughput on a link is given by
+%     buffer-size / latency.
+%     See https://support.microsoft.com/en-gb/help/823764/slow-performance-occurs-when-you-copy-data-to-a-tcp-server-by-using-a
+%     for Microsoft's discussion
 
 %!  tcp_fcntl(+Stream, +Action, ?Argument) is det.
 %
@@ -647,13 +675,13 @@ negotiate_socks_connection(Host:Port, StreamPair):-
 The C-layer generates exceptions of the  following format, where Message
 is extracted from the operating system.
 
-        error(socket_error(Message), _)
+        error(socket_error(Code, Message), _)
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 :- multifile
     prolog:error_message//1.
 
-prolog:error_message(socket_error(Message)) -->
+prolog:error_message(socket_error(_Code, Message)) -->
     [ 'Socket error: ~w'-[Message] ].
 prolog:error_message(socks_error(Error)) -->
     socks_error(Error).
