@@ -182,6 +182,12 @@ typedef struct idg_node
   unsigned	reevaluating : 1;	/* currently re-evaluating */
   unsigned	aborted : 1;		/* re-evaluation was aborted */
   int		falsecount;		/* Invalidate count */
+#ifdef O_TRIE_STATS
+  struct
+  { uint64_t	invalidated;		/* # times it was invalidated */
+    uint64_t	reevaluated;		/* # times it was re-evaluated */
+  } stats;
+#endif
 } idg_node;
 
 
@@ -202,5 +208,6 @@ COMMON(void)	tbl_push_delay(atom_t atrie, Word wrapper,
 COMMON(int)	answer_is_conditional(trie_node *answer);
 COMMON(void)	untable_from_clause(Clause cl);
 COMMON(void)	initTabling(void);
-
+COMMON(int)	idg_add_dyncall(Definition def, trie *ctrie,
+				term_t variant ARG_LD);
 #endif /*_PL_TABLING_H*/
