@@ -184,19 +184,17 @@ release_key(word key)
 
 
 trie *
-trie_create(void)
+trie_create(trie_allocation_pool *pool)
 { trie *trie;
 
-  if ( (trie = PL_malloc(sizeof(*trie))) )
+  if ( (trie = alloc_from_pool(pool, sizeof(*trie))) )
   { memset(trie, 0, sizeof(*trie));
     trie->magic = TRIE_MAGIC;
     trie->node_count = 1;		/* the root */
-
-    return trie;
-  } else
-  { PL_resource_error("memory");
-    return NULL;
+    trie->alloc_pool = pool;
   }
+
+  return trie;
 }
 
 
@@ -205,7 +203,7 @@ trie_destroy(trie *trie)
 { DEBUG(MSG_TRIE_GC, Sdprintf("Destroying trie %p\n", trie));
   trie->magic = TRIE_CMAGIC;
   trie_empty(trie);
-  PL_free(trie);
+  free_to_pool(trie->alloc_pool, trie, sizeof(*trie));
 }
 
 
@@ -277,22 +275,11 @@ static trie_node *
 new_trie_node(trie *trie, word key)
 { trie_node *n;
 
-  if ( trie->alloc_pool )
-  { if ( trie->alloc_pool->size+sizeof(trie_node) <= trie->alloc_pool->limit )
-    { ATOMIC_ADD(&trie->alloc_pool->size, sizeof(trie_node));
-    } else
-    { PL_resource_error("table_space");
-      return NULL;
-    }
-  }
-
-  if ( (n = PL_malloc(sizeof(*n))) )
+  if ( (n = alloc_from_pool(trie->alloc_pool, sizeof(*n))) )
   { ATOMIC_INC(&trie->node_count);
     memset(n, 0, sizeof(*n));
     acquire_key(key);
     n->key = key;
-  } else
-  { PL_resource_error("memory");
   }
 
   return n;
@@ -315,9 +302,7 @@ next:
 
   if ( dealloc )
   { ATOMIC_DEC(&trie->node_count);
-    if ( trie->alloc_pool )
-      ATOMIC_SUB(&trie->alloc_pool->size, sizeof(trie_node));
-    PL_free(n);
+    free_to_pool(trie->alloc_pool, n, sizeof(trie_node));
   } else
   { n->children.any = NULL;
   }
@@ -392,6 +377,25 @@ prune_node(trie *trie, trie_node *n)
 }
 
 
+#define VMASKBITS  (sizeof(unsigned)*8)
+#define VMASK_SCAN (0x1<<(VMASKBITS-1))
+
+static inline void
+update_var_mask(trie_children_hashed *hnode, word key)
+{ if ( tagex(key) == TAG_VAR )
+  { size_t vn = (size_t)(key>>LMASK_BITS); /* 1.. */
+    unsigned mask;
+
+    if ( vn < VMASKBITS )
+      mask = 0x1<<(vn-1);
+    else
+      mask = VMASK_SCAN;
+
+    ATOMIC_OR(&hnode->var_mask, mask);
+  }
+}
+
+
 static trie_node *
 insert_child(trie *trie, trie_node *n, word key ARG_LD)
 { for(;;)
@@ -407,27 +411,31 @@ insert_child(trie *trie, trie_node *n, word key ARG_LD)
 	{ if ( children.key->key == key )
 	  { return children.key->child;
 	  } else
-	  { trie_children_hashed *hnode = PL_malloc(sizeof(*hnode));
+	  { trie_children_hashed *hnode;
+
+	    if ( !(hnode=alloc_from_pool(trie->alloc_pool, sizeof(*hnode))) )
+	    { destroy_node(trie, new);
+	      return NULL;
+	    }
 
 	    hnode->type     = TN_HASHED;
 	    hnode->table    = newHTable(4);
-	    hnode->var_keys = 0;
+	    hnode->var_mask = 0;
 	    addHTable(hnode->table, (void*)children.key->key,
 				    children.key->child);
 	    addHTable(hnode->table, (void*)key, (void*)new);
-	    if ( tagex(children.key->key) == TAG_VAR )
-	      hnode->var_keys++;
-	    if ( tagex(new->key) == TAG_VAR )
-	      hnode->var_keys++;
+	    update_var_mask(hnode, children.key->key);
+	    update_var_mask(hnode, new->key);
 
 	    if ( COMPARE_AND_SWAP(&n->children.hash, children.hash, hnode) )
-	    { PL_free(children.any);		/* TBD: Safely free */
+	    {					/* TBD: Safely free */
+	      free_to_pool(trie->alloc_pool, children.any, sizeof(trie_children_key));
 	      new->parent = n;
 	      return new;
 	    }
 	    destroy_node(trie, new);
 	    destroyHTable(hnode->table);
-	    PL_free(hnode);
+	    free_to_pool(trie->alloc_pool, hnode, sizeof(*hnode));
 	    continue;
 	  }
 	}
@@ -437,8 +445,7 @@ insert_child(trie *trie, trie_node *n, word key ARG_LD)
 
 	  if ( new == old )
 	  { new->parent = n;
-	    if ( tagex(new->key) == TAG_VAR )
-	      ATOMIC_INC(&children.hash->var_keys);
+	    update_var_mask(children.hash, new->key);
 	  } else
 	  { destroy_node(trie, new);
 	  }
@@ -448,7 +455,12 @@ insert_child(trie *trie, trie_node *n, word key ARG_LD)
 	  assert(0);
       }
     } else
-    { trie_children_key *child = PL_malloc(sizeof(*child));
+    { trie_children_key *child;
+
+      if ( !(child=alloc_from_pool(trie->alloc_pool, sizeof(*child))) )
+      { destroy_node(trie, new);
+	return NULL;
+      }
 
       child->type  = TN_KEY;
       child->key   = key;
@@ -459,7 +471,7 @@ insert_child(trie *trie, trie_node *n, word key ARG_LD)
 	return child->child;
       }
       destroy_node(trie, new);
-      PL_free(child);
+      free_to_pool(trie->alloc_pool, child, sizeof(*child));
     }
   }
 }
@@ -891,7 +903,7 @@ PRED_IMPL("trie_new", 1, trie_new, 0)
 { PRED_LD
   trie *trie;
 
-  if ( (trie = trie_create()) )
+  if ( (trie = trie_create(NULL)) )
   { atom_t symbol = trie_symbol(trie);
     int rc;
 
@@ -1523,11 +1535,12 @@ unify_key(ukey_state *state, word key ARG_LD)
 
 
 typedef struct trie_choice
-{ union
-  { void *any;
-    TableEnum table;
-  } choice;
-  word key;
+{ TableEnum  table_enum;
+  Table      table;
+  unsigned   var_mask;
+  unsigned   var_index;
+  word       novar;
+  word       key;
   trie_node *child;
 } trie_choice;
 
@@ -1551,6 +1564,8 @@ typedef struct
   desc_tstate  buffer[64];	/* Quick buffer for stack */
 } descent_state;
 
+static int	advance_node(trie_choice *ch ARG_LD);
+
 static void
 init_trie_state(trie_gen_state *state, trie *trie)
 { state->trie = trie;
@@ -1569,8 +1584,8 @@ clear_trie_state(trie_gen_state *state)
   trie_choice *top = top_choice(state);
 
   for(; chp < top; chp++)
-  { if ( chp->choice.table )
-      freeTableEnum(chp->choice.table);
+  { if ( chp->table_enum )
+      freeTableEnum(chp->table_enum);
   }
 
   discardBuffer(&state->choicepoints);
@@ -1631,6 +1646,19 @@ get_key(trie_gen_state *state, descent_state *dstate, word *key ARG_LD)
 }
 
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Walk a step down the trie, adding  a   node  to the choice stack. If the
+term we are walking is instantiated and   the trie node does not contain
+variables we walk deterministically. Once we have a variable in the term
+we unify against or find a variable in  the trie dstate->prune is set to
+FALSE, indicating we must create a real choice.
+
+If a known input value is matched against   a  trie choice node and this
+node contains variables we create a choice   from the value and variable
+mask such that  we  perform  a  couple   of  hash  lookups  rather  than
+enumerating the entire table.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
 trie_choice *
 add_choice(trie_gen_state *state, descent_state *dstate, trie_node *node ARG_LD)
 { trie_children children = node->children;
@@ -1657,15 +1685,16 @@ add_choice(trie_gen_state *state, descent_state *dstate, trie_node *node ARG_LD)
 	     k == children.key->key ||
 	     tagex(children.key->key) == TAG_VAR ||
 	     IS_TRIE_KEY_POP(children.key->key) )
-	{ word key   = children.key->key;
+	{ word key = children.key->key;
 
 	  if ( tagex(children.key->key) == TAG_VAR )
 	    dstate->prune = FALSE;
 
 	  ch = allocFromBuffer(&state->choicepoints, sizeof(*ch));
-	  ch->key    = key;
-	  ch->child  = children.key->child;
-	  ch->choice.any = NULL;
+	  ch->key        = key;
+	  ch->child      = children.key->child;
+	  ch->table_enum = NULL;
+	  ch->table      = NULL;
 
 	  if ( IS_TRIE_KEY_POP(children.key->key) && dstate->compound )
 	  { desc_tstate dts;
@@ -1681,27 +1710,53 @@ add_choice(trie_gen_state *state, descent_state *dstate, trie_node *node ARG_LD)
 	  return NULL;
 	}
       case TN_HASHED:
-	if ( has_key && children.hash->var_keys == 0 )
-	{ trie_node *child;
+      { void *tk, *tv;
 
-	  if ( (child = lookupHTable(children.hash->table, (void*)k)) )
-	  { ch = allocFromBuffer(&state->choicepoints, sizeof(*ch));
-	    ch->key = k;
-	    ch->child = child;
-	    ch->choice.any = NULL;
-	  } else
-	    return NULL;
-	} else
-	{ void *k, *v;
+	if ( has_key )
+	{ if ( children.hash->var_mask == 0 )
+	  { trie_node *child;
 
-	  dstate->prune = FALSE;
-	  ch = allocFromBuffer(&state->choicepoints, sizeof(*ch));
-	  ch->choice.table = newTableEnum(children.hash->table);
-	  advanceTableEnum(ch->choice.table, &k, &v);
-	  ch->key   = (word)k;
-	  ch->child = (trie_node*)v;
+	    if ( (child = lookupHTable(children.hash->table, (void*)k)) )
+	    { ch = allocFromBuffer(&state->choicepoints, sizeof(*ch));
+	      ch->key        = k;
+	      ch->child	     = child;
+	      ch->table_enum = NULL;
+	      ch->table      = NULL;
+
+	      return ch;
+	    } else
+	      return NULL;
+	  } else if ( children.hash->var_mask != VMASK_SCAN )
+	  { dstate->prune = FALSE;
+
+	    DEBUG(MSG_TRIE_GEN,
+		  Sdprintf("Created var choice 0x%x\n", children.hash->var_mask));
+
+	    ch = allocFromBuffer(&state->choicepoints, sizeof(*ch));
+	    ch->table_enum = NULL;
+	    ch->table      = children.hash->table;
+	    ch->var_mask   = children.hash->var_mask;
+	    ch->var_index  = 1;
+	    ch->novar      = k;
+	    if ( advance_node(ch PASS_LD) )
+	    { return ch;
+	    } else
+	    { state->choicepoints.top = (char*)ch;
+	      ch--;
+	      return NULL;
+	    }
+	  }
 	}
+					/* general enumeration */
+	dstate->prune = FALSE;
+	ch = allocFromBuffer(&state->choicepoints, sizeof(*ch));
+	ch->table = NULL;
+	ch->table_enum = newTableEnum(children.hash->table);
+	advanceTableEnum(ch->table_enum, &tk, &tv);
+	ch->key   = (word)tk;
+	ch->child = (trie_node*)tv;
 	break;
+      }
       default:
 	assert(0);
         return NULL;
@@ -1727,16 +1782,35 @@ descent_node(trie_gen_state *state, descent_state *dstate, trie_choice *ch ARG_L
 
 
 static int
-advance_node(trie_choice *ch)
-{ if ( ch->choice.table )
+advance_node(trie_choice *ch ARG_LD)
+{ if ( ch->table_enum )
   { void *k, *v;
 
-    if ( advanceTableEnum(ch->choice.table, &k, &v) )
+    if ( advanceTableEnum(ch->table_enum, &k, &v) )
     { ch->key   = (word)k;
       ch->child = (trie_node*)v;
 
       return TRUE;
     }
+  } else if ( ch->table )
+  { if ( ch->novar )
+    { if ( (ch->child=lookupHTable(ch->table, (void*)ch->novar)) )
+      { ch->key = ch->novar;
+	ch->novar = 0;
+	return TRUE;
+      }
+    }
+    for( ; ch->var_index && ch->var_index < VMASKBITS; ch->var_index++ )
+    { if ( (ch->var_mask & (0x1<<(ch->var_index-1))) )
+      { word key = ((((word)ch->var_index))<<LMASK_BITS)|TAG_VAR;
+
+	if ( (ch->child=lookupHTable(ch->table, (void*)key)) )
+	{ ch->key = key;
+	  ch->var_index++;
+	  return TRUE;
+	}
+      }
+    }
   }
 
   return FALSE;
@@ -1749,11 +1823,11 @@ next_choice0(trie_gen_state *state, descent_state *dstate ARG_LD)
   trie_choice  *ch = top_choice(state)-1;
 
   while(ch >= btm)
-  { if ( advance_node(ch) )
+  { if ( advance_node(ch PASS_LD) )
       return descent_node(state, dstate, ch PASS_LD);
 
-    if ( ch->choice.table )
-      freeTableEnum(ch->choice.table);
+    if ( ch->table_enum )
+      freeTableEnum(ch->table_enum);
 
     state->choicepoints.top = (char*)ch;
     ch--;
@@ -2501,6 +2575,40 @@ set_trie_clause_general_undefined(Clause clause)
   }
 }
 
+
+		 /*******************************
+		 *	     ALLOCATION		*
+		 *******************************/
+
+void *
+alloc_from_pool(trie_allocation_pool *pool, size_t bytes)
+{ void *mem;
+
+  if ( pool )
+  { if ( pool->size+bytes <= pool->limit )
+    { ATOMIC_ADD(&pool->size, bytes);
+    } else
+    { PL_resource_error("table_space");
+      return NULL;
+    }
+  }
+
+  if ( (mem=malloc(bytes)) )
+    return mem;
+
+  PL_resource_error("memory");
+  return NULL;
+}
+
+void
+free_to_pool(trie_allocation_pool *pool, void *mem, size_t bytes)
+{ free(mem);
+
+  if ( pool )
+    ATOMIC_SUB(&pool->size, bytes);
+}
+
+
 		 /*******************************
 		 *      PUBLISH PREDICATES	*
 		 *******************************/
